
package RRDutils;

use warnings;
use strict;

BEGIN {
   use Exporter ();
    our ( $VERSION , @ISA , @EXPORT , @EXPORT_OK , %EXPORT_TAGS );

   $VERSION="2.2";
   @ISA = qw(Exporter);   
   @EXPORT = qw(
      $rrdtool
      changeDefs $confdir $datadir $graphdir
      getHosts
      );
   @EXPORT_OK = qw(
      getStep getStart getDSs getRRAs
      $figSize %graphInterval @colors
      getIntervals getGraphs getLabels
      );
   %EXPORT_TAGS = (
      Create => [ qw(getStep getStart getDSs getRRAs) ]
    , Graph => [ qw(
        $figSize %graphInterval @colors
        getIntervals getGraphs getLabels
        ) ]
      );
   }

our ( $rrdtool , $confdir , $datadir , $graphdir );

$rrdtool = "%%RRDTOOLDIR%%/rrdtool";

my $rootdir = "/var/lib/rrd";
$confdir = "$rootdir/conf";
$datadir = "$rootdir/data";
$graphdir = "$rootdir/graphs";

my $defStep = 300;
my @defGraph = ( 300, 90 );

our ( $figSize , %graphInterval , @colors );

$figSize="--width $defGraph[0] --height $defGraph[1]";

%graphInterval = (
   'daily' => -86400 ,
   'weekly' => -604800 ,
   'monthly' => -2592000 ,
   'quarterly' => -10368000 ,
   'yearly' => -41472000 );

@colors= qw(ff0000 00ff00 0000ff aa0000 00aa00 0000aa
   880000 008800 000088 660000 006600 000066
   ff0000 00ff00 0000ff ff0000 00ff00 0000ff
   ff0000 00ff00 0000ff ff0000 00ff00 0000ff
   ff0000 00ff00 0000ff ff0000 00ff00 0000ff
   ff0000 00ff00 0000ff ff0000 00ff00 0000ff
   ff0000 00ff00 0000ff ff0000 00ff00 0000ff
   ff0000 00ff00 0000ff ff0000 00ff00 0000ff);


sub changeDefs($) {
   my $conf = shift;
   return unless open CFG , "$confdir/$conf";
   while (<CFG>) {
      next if /^\s*#/;
      if ( /^DataDir\s+(.+)$/ ) {
         $datadir=$1;
         }
      if ( /^GraphDir\s(.+)$/ ) {
         $graphdir=$1;
         }
      if ( /^DefGraph\s+([0-9]+)\s+([0-9]+)$/ ) {
         @defGraph = ( $1 , $2 );
         $figSize="--width $defGraph[0] --height $defGraph[1]";
         }
      }
   close CFG;
   }

sub getStep($) {
   my $conf = shift;
   my $step;
   return unless open CFG , "$confdir/$conf";
   while (<CFG>) {
      next if /^\s*#/;
      if ( /^BaseStep\s+([0-9]*)/ ) {
         warn "WARNING : base step redefined\n" if $step;
         $step=$1;
         }
      }
   close CFG;
   $step = $defStep unless $step;
   return $step;
   }

sub getStart($) {
   my $conf = shift;
   my $start;
   return unless open CFG , "$confdir/$conf";
   while (<CFG>) {
      next if /^\s*#/;
      if ( /^Start\s+(.*)/ ) {
         warn "WARNING : Start time for RRDB defined twice" if $start;
         $start=$1;
         }
      }
   return $start;
   }

sub getHosts($) {
   my ( $conf , $tag ) = split /\./ , shift ;
   $tag = "" unless $tag;
   my @hosts;
   return unless open CFG , "$confdir/$conf";
   while (<CFG>) {
      next if /^\s*#/;
      if ( s/^H$tag\s+// ) {
         push @hosts , split;
         }
      if ( s/^T\s+// ) {
         push @hosts , [ split ];
         }
      }
   close CFG;
   return @hosts;
   }


sub getDSs($) {
   my $conf = shift;
   my %types = (
      'G' => 'GAUGE' ,
      'C' => 'COUNTER' ,
      'D' => 'DERIVE' ,
      'A' => 'ABSOLUTE' );
   my ( @DSs , $label );
   my $step = getStep $conf;
   return unless open CFG , "$confdir/$conf";
   $_ = <CFG>; chomp;
   while ( 1 ) {
      last if eof CFG;
      if ( /^[^#]/ && /^([^\s\[]*)?\[([^\s\/]+)\](\s+(.*))?$/ ) {
         $label = $2;
         my ( $type, $beat, $min, $max ) = ( "COUNTER", 3 , "U", "U" );
         $_ = <CFG>; $_ =~ s/\s*$//;
         $_ =~ s/\s*$//;
         if ( s/^((C|G|D|A)\s?)?\s*([0-9\.]+)?(\s+((U|-?[0-9\.]+)\s+)?(U|-?[0-9\.]+))?$// ) {
            $type = $types{$2} if $2;
            $beat = $3 if $3;
            $min = $6 if $6;
            $max = $7 if $7;
            unless ( eof CFG ) { $_ = <CFG> ; chomp }
            }
         push @DSs , "$label:$type:" . $beat * $step . ":$min:$max";
      } else {
         last if eof CFG;
         $_ = <CFG>; chomp;
         }
      }
   close CFG;

   return @DSs;
   }

sub getRRAs($) {
   my $conf = shift;
   my @RRAs;
   my $TimeStep = 1;
   return unless open CFG , "$confdir/$conf";
   while ( <CFG> ) {
      next if /^\s*#/;
      if ( /^TimeStep\s+(.*)/ ) {
         $TimeStep *= $1;
         next;
         }
      if ( s/^(CF\s+)?(AVERAGE|MAX|MIN|LAST)// ) { #(.*)(\s+([0-9]+))?$/ ) {
         my $CF = $2;
         my ( $xff , $step , $rows ) = ( 0.5 , 1 , 192 );
         /(\s+(0|0?\.[0-9]+)?\s*([1-9][0-9]*)?)?\s+([0-9]+)$/;
         $xff = $2 if $2;
         $step = $3 if $3;
         $rows = $4 if $4;
         push @RRAs , "$CF:$xff:" . $step * $TimeStep . ":$rows";
         next;
         }
      }
   close CFG;
   @RRAs = ( "AVERAGE:0.5:1:2048" ) unless @RRAs;
   return @RRAs;
   }


sub getIntervals($) {
   my $conf = shift;
   return unless open CFG , "$confdir/$conf";
   while (<CFG>) {
      next if /^\s*#/;
      if ( /^I\s+([^\s]+)\s+(\d+)/ ) {
         $graphInterval{$1} = "-$2";
         }
      }
   close CFG;
   }

sub getSize($) {
   my $size = shift;
   $size =~ /^(.*)x(.*)/;
   my $xsize = $1 || $defGraph[0];
   my $ysize = $2 || $defGraph[1];
   return " --width $xsize --height $ysize ";
   }

sub getGraphs($) {
   my $conf = shift;
   my $opts = "";
   my %graphs;
   return unless open CFG , "$confdir/$conf";
   while (<CFG>) {
      next if /^\s*#/;
      if ( s/^O(=?)\s+// ) {
         chomp;
         $opts = "$_" if $1;
         $opts .= " $_";
         }
      if ( s/^(P(S|H)?)(_?([a-z_]+))?(\s+([0-9]*x[0-9]*))?\s+([a-z]+)\s+([^\\]*)(\s+(\\))?$/$8/ ) {
         my @vars=split;
         my $size = "";
         $size=getSize($6) if $6;
         my $idx = join "." , @vars;
         $idx = $4 if $4;
         $idx .= ".$7";
         $graphs{$idx}{'type'} = $1;
         $graphs{$idx}{'size'} = $size;
         $graphs{$idx}{'period'} = $7;
         $graphs{$idx}{'opts'} = $opts;
         $graphs{$idx}{'ds'} = \@vars ;
         my @print = ();
         if ( $10 && $10 eq "\\" ) {
            $_ = <CFG>; chomp;
            @print = split;
            }
         $graphs{$idx}{'print'} = \@print;
         }
      }
   close CFG;
   return %graphs;
   }


sub getLabels($) {
   my $conf = shift;
   my %labels;
   return unless open CFG , "$confdir/$conf";
   while (<CFG>) {
      next if /^\s*#/;
      if ( /^([^\s\[]*)?\[([^\s\/]+)\](\s+(.*))?$/ ) {
         $labels{$2} = $4 || $1 || $2;
         $labels{$2} =~ s/\s*$//;
         }
      }
   close CFG;
   return %labels;
   }

1;
